/****************************************************************
**
** Attal : Lords of Doom
**
** propertiesBuilding.cpp
** Defines properties box for buildings on map
**
** Version : $Id: propertiesBuilding.cpp,v 1.12 2006/02/23 21:06:52 audoux Exp $
**
** Author(s) : Sardi Carlo - Pascal Audoux
**
** Date : 22/04/2004
**
** Licence :    
**	This program is free software; you can redistribute it and/or modify
**   	it under the terms of the GNU General Public License as published by
**     	the Free Software Foundation; either version 2, or (at your option)
**      any later version.
**
**	This program is distributed in the hope that it will be useful,
** 	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
****************************************************************/

#include "propertiesBuilding.h"

// generic include files
// include files for QT
#include <QComboBox>
#include <QHBoxLayout>
#include <QLabel>
#include <QLayout>
#include <QListWidget>
#include <QPushButton>

// application specific include files
#include "libCommon/genericLord.h"
#include "libCommon/genericPlayer.h"
#include "libCommon/dataTheme.h"
#include "libClient/gui.h"
#include "libCommon/action.h"

extern DataTheme DataTheme;


//
// ----- propertiesBuilding -----
//

PropertiesBuilding::PropertiesBuilding( QList<GenericPlayer *> * players, QWidget * parent, const char * /* name */ )
	: QDialog( parent, Qt::Dialog )
{
        setWindowTitle( tr( "building properties" ) );
	_building = 0;

	QLabel * ownerLabel = new QLabel( this );
	ownerLabel->setText( tr( "Owner:" ) );

	_name = new QLabel( this );
	
	_ownerList = new QComboBox( this );
	refresh( players );

	_listBuilding = new QListWidget( this );

	QPushButton * pb = new QPushButton( this );
	pb->setText( tr( "Ok" ) );
	FIXEDSIZE( pb );

	QHBoxLayout * layH1 = new QHBoxLayout();
	layH1->setSpacing( 5 );
	layH1->addWidget( ownerLabel );
	layH1->addWidget( _ownerList, 1 );

	QHBoxLayout * layH2 = new QHBoxLayout();
	layH2->setSpacing( 5 );
	layH2->addWidget( _listBuilding, 1 );

	QHBoxLayout * layH = new QHBoxLayout();
	layH->setMargin( 5 );
	layH->addStretch( 1 );
	layH->addWidget( pb );

	QVBoxLayout * layout = new QVBoxLayout( this );
	layout->setMargin( 5 );
	layout->setSpacing( 5 );
	layout->addLayout( layH1 );
	layout->addStretch( 1 );
	layout->addWidget( _name );
	layout->addLayout( layH2, 1 );
	layout->addLayout( layH );
	layout->activate();

	connect( pb, SIGNAL( clicked() ), SLOT( slot_accept() ) );
	connect( _ownerList, SIGNAL( activated( int ) ), SLOT( slot_owner( int ) ) );
}

void PropertiesBuilding::refresh( QList<GenericPlayer *> * players )
{
	_players = players;
	_ownerList->clear();
	_ownerList->insertItem( tr( "None" ) );
	
	for( int i = 0; i < players->count(); i++ ) {
		GenericPlayer * player = players->at( i );
		if( player ) {
			_ownerList->insertItem( player->getName() );
		}
	}
}


void PropertiesBuilding::slot_accept()
{
	accept();
}

void PropertiesBuilding::slot_owner( int num )
{
	if( _building->getOwner() ) {
		_building->getOwner()->removeBuilding( _building );
	}
	if( num == 0 ) {
		_building->setOwner( 0 );
	} else {
		_building->setOwner( _players->at( num-1 ) );
		_players->at( num-1 )->addBuilding( _building );
	}
}

void PropertiesBuilding::init( GenericBuilding * building )
{
	_building = building;
	_listBuilding->clear();
	_name->setText( _building->getName() );

	for( int type = Action::DATE; type<= Action::NEXTTIME; type++ ){
		QList<Action *>  action = _building->getActionList((Action::ActionType) type);
		if( action.count() ) {
			QString tmp = "";
			switch( (Action::ActionType) type)  {
				case Action::DATE:
					tmp = "Date ";
					break;
				case Action::FIRSTTIME:
					tmp = "First time ";
					break;
				case Action::FIRSTTIMELORD:
					tmp = "First time lord ";
					break;
				case Action::NEXTTIME:
					tmp = "Next time ";
					break;
				default:
					break;
			}
			if( tmp != "" ) {
				_listBuilding->addItem( tmp );
			}
		}
		for( int i = 0; i < action.count(); i++ ) {
			for( int j = 0; j < action.at(i)->getElementaryNumber(); j++ ) {
				ElementaryAction * elementary = action.at(i)->getElementaryAction( j );
				QString tmp = "";
				switch( elementary->getType() ) {
					case ElementaryAction::ATTACK:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Attack ");
						break;
					case ElementaryAction::DEFENSE:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Defense ");
						break;
					case ElementaryAction::POWER:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Power ");
						break;
					case ElementaryAction::KNOWLEDGE:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Knowledge ");
						break;
					case ElementaryAction::MOVE:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Move ");
						break;
					case ElementaryAction::MAXMOVE:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("MaxMove ");
						break;
					case ElementaryAction::TECHNICPOINT:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("TechPoint ");
						break;
					case ElementaryAction::MAXTECHNICPOINT:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("MaxTechPoint ");
						break;
					case ElementaryAction::MORALE:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Morale ");
						break;
					case ElementaryAction::LUCK:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Luck ");
						break;
					case ElementaryAction::VISION:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Vision ");
						break;
					case ElementaryAction::EXPERIENCE:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Experience ");
						break;
					case ElementaryAction::CHARISMA:
						tmp = _building->getName().append(" add %1 ").arg(elementary->getCoeff()).append("Charisma ");
						break;
					case ElementaryAction::SKILL:
						break;
					case ElementaryAction::ARTEFACT:
						break;
					case ElementaryAction::UNIT:
						break;				
					case ElementaryAction::RESSOURCE:
						{
							QString tmp1 =DataTheme.resources.getRessource(elementary->getArg());
							tmp = _building->getName().append(" give %1").arg(elementary->getCoeff()).append(" ").append(tmp1);
						}

						break;
					case ElementaryAction::RANDRESSOURCE:
						{
							QString tmp1 = DataTheme.resources.getRessource(elementary->getArg());
							tmp = _building->getName().append(" give between 0 and %1").arg(elementary->getCoeff()*2).append(" ").append(tmp1);
						}
						break;
					default:
						break;
				}
				if( tmp != "" ) {
					_listBuilding->addItem( tmp );
				}
			}
		}
	}

	if( _building->getOwner() ) {
		_ownerList->setCurrentItem( _building->getOwner()->getNum()+1 );
	} else {
		_ownerList->setCurrentItem( 0 );
	}
}




