/****************************************************************
**
** Attal : Lords of Doom
**
** engine.h
** the game engine !
**
** Version : $Id: engine.h,v 1.38 2005/12/30 18:44:49 lusum Exp $
**
** Author(s) : Pascal Audoux - Sardi Carlo
**
** Date : 03/10/2000
**
** Licence :
**	This program is free software; you can redistribute it and/or modify
**   	it under the terms of the GNU General Public License as published by
**     	the Free Software Foundation; either version 2, or (at your option)
**      any later version.
**
**	This program is distributed in the hope that it will be useful,
** 	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
****************************************************************/


#ifndef ENGINE_H
#define ENGINE_H


// generic include files
// include files for QT
#include <QObject>
#include <QList>
// application specific includes
#include "libCommon/fightResultStatus.h"
#include "libCommon/genericBase.h"
#include "libCommon/genericMap.h"
#include "libCommon/genericLord.h"
#include "libCommon/genericPlayer.h"

#include "libServer/attalServer.h"
#include "libServer/fightEngine.h"
#include "libServer/gameData.h"

class Campaign;
class GenericBonus;
class GenericMapCreature;
class QuestionManager;
class ScenarioParser;
class TavernManager;

/*              ------------------------------
 *                         Engine
 *              ------------------------------ */


class Engine : public QObject, public GameData
{
	Q_OBJECT
public:
	/** Constructor */
	Engine( AttalServer * serv );

	/** Start game (scenario) */
	void startGame();
	
	/** Start game (scenario) */
	void startCampaign();
	
	void cancelCampaign();

	/** Load a game (scenario) */
	bool loadGame( const QString & filename , bool silent);
	
	/** Load a game (scenario) */
	bool loadCampaign( const QString & filename );

	/** Save the game */
	bool saveGame( const QString & filename );

	/** End the game */
	void endGame();

	/** Start a fight */
	void startFight();
	void startFight( int lordAttack, GenericLord *lordDefense );
	void startFight( int lordAttack, GenericMapCreature * creature );

	/** End the fight */
	void endFight();

	/** Reinit game engine */
	void reinit();
	
	int getNumFillPlayers() { return _tmpPlay > 0 ? _tmpPlay : 0 ; }

	/** Activate the next player */
	void nextPlayer();
	
	Campaign * getCampaign() { return _campaign; }
	
	void setFinished(bool finish) {_isFinished = finish;}

	enum State {
		NOT_PLAYING,
		IN_GAME,
		IN_QUESTION,
		IN_FIGHT
	};

	State getState() { return _state; }

public slots:
	/** Slot who reads socket */
	void slot_readSocket( int );

	/** Slot for managing new players */
	void slot_newPlayer( AttalPlayerSocket * player );

	/** Slot for removing a player disconnected */
	void slot_endConnection( QString name );

	/** Slot catching end of fight */
	void slot_endFight( FightResultStatus result );

signals:
	void sig_newPlayer( AttalPlayerSocket * player );
	
	void sig_result(bool);
	
	void sig_endGame();

private:
	enum CreatureAction {
		CreatureJoin,
		CreatureMercenary,
		CreatureFlee,
		CreatureFight
	};

	CreatureAction computeCreatureAction( GenericMapCreature * creature, double ratio );

	void checkMainQuest();

	void stateNotPlaying( int num );
	void stateInGame( int num );
	void stateInQuestion( uint num );
	void stateInFight( int num );
	
	void handleMessage();
	void handleAnswer();
	void handleInGameMvt( int num );
	void handleInGameExchange();
	void handleInGameModif();
	void handleInGameModifBase();
	void handleInGameModifBaseBuilding();
	void handleInGameModifBaseUnit();
	void handleInGameModifBaseMarket();
	void handleInGameModifLord();
	void handleInGameModifLordGarrison();
	void handleInGameModifLordUnit();
	void handleInGameModifLordBuy();
	void handleInGameTurn();
	
	void handleGameTavern();

	void newDay();

	void handleAnswerCreatureJoin();
	void handleAnswerCreatureMercenary();
	void handleAnswerCreatureFlee();
	
	// enter into building (if return == true)
	bool handleBuildingEnter( GenericBuilding * building, GenericLord * lord );
	void handleBuildingAction( Action * action, GenericLord * lord );

	void handleCreatTurn( GenericPlayer * player, GenericLord * lord );
	void handleBaseTurn( GenericPlayer * player, GenericBase * base );
	void handleBuildingTurn( GenericPlayer * player, GenericBuilding * building );
	void handleBuildingAction( Action * action, GenericPlayer * player );
	void handlePopulation( GenericPlayer * player );

	bool handleOneMove( GenericLord * movingLord, GenericCell * destCell, int num );
	void movingOnLord( GenericLord * movingLord, GenericCell * destCell );
	void movingOnEvent( GenericLord * movingLord, GenericCell * destCell );
	void movingOnArtefact( GenericLord * movingLord, GenericCell * destCell );
	void movingOnBonus( GenericLord * movingLord, GenericCell * destCell );
	void movingOnBonusResource( GenericBonus * bonus );
	void movingOnBonusPrimSkill( GenericLord * movingLord, GenericBonus * bonus );
	void movingOnChest( GenericLord * movingLord, GenericCell * destCell );
	void movingOnBuilding( GenericLord * movingLord, GenericCell * destCell, int num );
	void movingOnBase( GenericLord * movingLord, GenericCell * destCell, int num );
	void movingOnCreature( GenericLord * movingLord, GenericCell * destCell );
	void movingOnFreeCell( GenericLord * movingLord, GenericCell * destCell, int num );

	uint getAlivePlayersNumber();
	void exchangeUnits();
	void exchangeArtefact();
	void exchangeBaseUnits();
	bool enoughPlayers();

	int getRandResource( int coeff);
	void updatePlayerPrices( GenericPlayer * player);

	void updateMapVision( QList<GenericCell *> & removed, QList<GenericCell *> & added );
	void updateCellVision( GenericCell * cell );

	void manageIncreaseExperience( GenericLord * lord, uint experience );

	bool _isCreature;
	bool _isFinished;
	int _tmpPlay;

	AttalServer * _server;
	FightEngine * _fight;
	State _state;
	GenericPlayer * _currentPlayer;
	int _counter;
	Campaign * _campaign;

	QuestionManager * _question;
	TavernManager * _tavern;
};

#endif // ENGINE_H

















