/****************************************************************
**
** Attal : Lords of Doom
**
** serverInterface.h
** interface for the server
**
** Version : $Id: serverInterface.h,v 1.24 2006/02/10 21:07:40 audoux Exp $
**
** Author(s) : Pascal Audoux - Sardi Carlo
**
** Date : 01/11/2000
**
** Licence :
**	This program is free software; you can redistribute it and/or modify
**   	it under the terms of the GNU General Public License as published by
**     	the Free Software Foundation; either version 2, or (at your option)
**      any later version.
**
**	This program is distributed in the hope that it will be useful,
** 	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
****************************************************************/


#ifndef SERVERINTERFACE_H
#define SERVERINTERFACE_H

// include files for QT
#include <QCloseEvent>
#include <QDialog>
#include <QEvent>
#include <QLineEdit>
#include <QList>
#include <QMainWindow>
#include <QProcess>
#include <QPushButton>
#include <QRadioButton>
#include <QString>
#include <QVector>


// application specific includes
#include "libServer/attalServer.h"

#include "server/engine.h"

class QAction;
class QButtonGroup;
class QGroupBox;
class QLineEdit;
class QListWidget;
class QSignalMapper;
class QTreeWidget;

class ServerWidget;
class ConfigConnection;

/*              ------------------------------
 *                      ServerInterface
 *              ------------------------------ */


/** comment for the class */
class ServerInterface : public QMainWindow
{
	Q_OBJECT
public:
	/** Constructor */
	ServerInterface();
	
	~ServerInterface();

	bool initServer();	

	void loadCampaign( const QString & fileName );
	
public slots:
	/** Slot for managing 'actions' */
	void slot_action( int num );

	/** Slot managing the status bar */
	void slot_status( QString text );

	/** Slot for ending game */
	void slot_stop();

	/** Slot for loading scenario */
	void slot_load( QString );

	/** Slot for saving game */
	void slot_save();
	
	void slot_result(bool);
	
	void slot_endGame();

	void slot_started();
	
	void slot_ready();
	
	bool fillWithAI(QString filename);
	
protected:
	void closeEvent(QCloseEvent* ce);
	
	virtual void changeEvent ( QEvent * e );

private:
	enum MENU_ACTIONS {
		ACTION_LOADSCENARIO,
		ACTION_LOADCAMPAIGN,
		ACTION_LOADGAME,
		ACTION_SAVE,
		ACTION_END,
		ACTION_QUIT,
		
		ACTION_FILLAI,
		ACTION_ADDAI,
		ACTION_AIDBG,
		ACTION_HIDE,
		
		NB_ACTIONS
	};
		
	/** define actions */
	void initActions();

	/** Define menus */
	void initMenuBar();

	/** Define statusBar */
	void initStatusBar();

	/** Add new action */
	QAction * ServerInterface::addAction( const QString & label, MENU_ACTIONS id, QSignalMapper * sigmap );
	
	void addAI(bool hide);
	
	bool killAI();

	AttalServer * _server;
	Engine * _engine;
	ServerWidget * _widget;
	ConfigConnection * _config;

	uint _actScen;

	QVector<QAction *> _actions;
	QList<QProcess *> _proclist;
	bool _aiDbg, _hide, _inLoad, _winner, _ready;
};

/** radio button allowing to choose a file on the disk */
class ChooseFileWidget : public QWidget
{
	Q_OBJECT
public:
	/** Constructor */
	ChooseFileWidget( QWidget * parent = 0, const char * name = 0 );

	/** Return file in the lineedit */
	QString getText() { return _edit->text(); }

public slots:
	/** Slot for choosing file */
	void slot_choose();

	/** Toggle radio button */
	void slot_toggle( bool );

private:
	QLineEdit * _edit;
	QPushButton * _choose;
};

/** Main widget for the server */
class ServerWidget : public QWidget
{
	Q_OBJECT
public:
	/** Constructor */
	ServerWidget( QWidget * parent = 0, const char * name = 0 );

	/** Toggle buttons state as game started or not */
	void setGameLoaded( bool b );

	/** returns filename to load */
	QString getFilename();

public slots:
	/** Slot for start button */
	void slot_start();

	/** Slot for handling new players */
	void slot_newPlayer( AttalPlayerSocket * player );

	/** slot for handling disconnected players */
	void slot_endConnection( QString name );

signals:
	/** Signal for buttopn 'End' */
	void sig_stop();

	/** Signal for button 'Load' */
	void sig_load( QString );

	/** Signal for button 'Save' */
	void sig_save();

private:

	QButtonGroup * _group;
	QGroupBox * _groupWidget;
	QPushButton * _butStart, * _butStop, * _butSave;
	QRadioButton * _radio1, * _radio2, * _radio3;
	ChooseFileWidget * _choose;

	QTreeWidget * _playerList;
	bool _loaded;
};

/** Dialog for configuring connection to server */
class ConfigConnection : public QDialog
{
	Q_OBJECT
public:
	/** Constructor */
	ConfigConnection( QWidget * parent = 0 );

	/** Setting hostname */
	void setHost( QString host ) { _host->setText( host ); }

	/** Getting hostname */
	QString getHost() { return _host->text(); }

	int getPort() { return _port->text().toInt(); }

	/** Setting port number */
	void setPort( int port ) { _port->setText( QString::number( port ) ); }

	void accept() { QDialog::accept(); }

private:
	QLineEdit * _host, * _port;
};

#define FIXEDSIZE( wid ) (wid)->setFixedSize( (wid)->sizeHint() )



#endif // SERVERINTERFACE_H
